@@ -53,18 +53,6 @@
  *            on SIGUSR2 and sigaltstack in Crossfire).
  * 2008-01-21 Disable CFI usage on anything but GNU/Linux.
  * 2008-03-02 Switched to 2-clause BSD license with GPL exception.
- * 2008-04-04 New (but highly unrecommended) pthreads backend.
- * 2008-04-24 Reinstate CORO_LOSER (had wrong stack adjustments).
- * 2008-10-30 Support assembly method on x86 with and without frame pointer.
- * 2008-11-03 Use a global asm statement for CORO_ASM, idea by pippijn.
- * 2008-11-05 Hopefully fix misaligned stacks with CORO_ASM/SETJMP.
- * 2008-11-07 rbp wasn't saved in CORO_ASM on x86_64.
- *            introduce coro_destroy, which is a nop except for pthreads.
- *            speed up CORO_PTHREAD. Do no longer leak threads either.
- *            coro_create now allows one to create source coro_contexts.
- *            do not rely on makecontext passing a void * correctly.
- *            try harder to get _setjmp/_longjmp.
- *            major code cleanup/restructuring.
  */
 
 #ifndef CORO_H
@@ -118,10 +106,6 @@
  *    Handcoded assembly, known to work only on a few architectures/ABI:
  *    ELF Linux x86 && amd64 when gcc is used and optimisation is turned on.
  *
- * -DCORO_PTHREAD
- *
- *    Use the pthread API. You have to provide <pthread.h> and -lpthread.
- *
  * If you define neither of these symbols, coro.h will try to autodetect
  * the model. This currently works for CORO_LOSER only. For the other
  * alternatives you should check (e.g. using autoconf) and define the
@@ -145,14 +129,8 @@ typedef struct coro_context coro_context;
  * uninitialised coro_context, it expects a pointer to the entry function
  * and the single pointer value that is given to it as argument.
  *
- * Allocating/deallocating the stack is your own responsibility.
- *
- * As a special case, if coro, arg, sptr and ssize are all zero,
- * then an "empty" coro_contetx will be created that is suitable
- * as an initial source for coro_transfer.
- *
- * This function is not reentrant, but putting a mutex around it
- * will work.
+ * Allocating/deallocating the stack is your own responsibility, so there is
+ * no coro_destroy function.
  */
 void coro_create (coro_context *ctx, /* an uninitialised coro_context */
                   coro_func coro,    /* the coroutine code to be executed */
@@ -164,22 +142,8 @@ void coro_create (coro_context *ctx, /* an uninitialised coro_context */
  * The following prototype defines the coroutine switching function. It is
  * usually implemented as a macro, so watch out.
  *
- * This function is thread-safe and reentrant.
+void coro_transfer(coro_context *prev, coro_context *next);
  */
-#if 0
-void coro_transfer (coro_context *prev, coro_context *next);
-#endif
-
-/*
- * The following prototype defines the coroutine destroy function. It is
- * usually implemented as a macro, so watch out. It also serves
- * no purpose unless you want to use the CORO_PTHREAD backend.
- *
- * This function is thread-safe and reentrant.
- */
-#if 0
-void coro_destroy (coro_context *ctx);
-#endif
 
 /*
  * That was it. No other user-visible functions are implemented here.
@@ -189,8 +153,7 @@ void coro_destroy (coro_context *ctx);
 
 #if !defined(CORO_LOSER) && !defined(CORO_UCONTEXT) \
     && !defined(CORO_SJLJ) && !defined(CORO_LINUX) \
-    && !defined(CORO_IRIX) && !defined(CORO_ASM) \
-    && !defined(CORO_PTHREAD)
+    && !defined(CORO_IRIX) && !defined(CORO_ASM)
 # if defined(WINDOWS)
 #  define CORO_LOSER 1 /* you don't win with windoze */
 # elif defined(__linux) && (defined(__x86) || defined (__amd64))
@@ -215,7 +178,6 @@ struct coro_context {
 };
 
 # define coro_transfer(p,n) swapcontext (&((p)->uc), &((n)->uc))
-# define coro_destroy(ctx) (void *)(ctx)
 
 #elif CORO_SJLJ || CORO_LOSER || CORO_LINUX || CORO_IRIX
 
@@ -223,60 +185,26 @@ struct coro_context {
 #  define _GNU_SOURCE /* for linux libc */
 # endif
 
-# if !CORO_LOSER
-#  include <unistd.h>
-# endif
-
-/* solaris is hopelessly borked, it expands _XOPEN_UNIX to nothing */
-# if __sun
-#  undef _XOPEN_UNIX
-#  define _XOPEN_UNIX 1
-# endif
-
 # include <setjmp.h>
 
 struct coro_context {
-#if _XOPEN_UNIX > 0 || CORO_LOSER
   jmp_buf env;
-#else
-  sigjmp_buf env;
-#endif
 };
 
-# if _XOPEN_UNIX > 0
-#  define coro_transfer(p,n) do { if (!  _setjmp ((p)->env   ))   _longjmp ((n)->env, 1); } while (0)
-# elif CORO_LOSER
-#  define coro_transfer(p,n) do { if (!   setjmp ((p)->env   ))    longjmp ((n)->env, 1); } while (0)
+# if CORO_LINUX || (_XOPEN_SOURCE >= 600)
+#  define coro_transfer(p,n) do { if (!_setjmp ((p)->env)) _longjmp ((n)->env, 1); } while (0)
 # else
-#  define coro_transfer(p,n) do { if (!sigsetjmp ((p)->env, 0)) siglongjmp ((n)->env, 1); } while (0)
+#  define coro_transfer(p,n) do { if (!setjmp  ((p)->env)) longjmp  ((n)->env, 1); } while (0)
 # endif
 
-# define coro_destroy(ctx) (void *)(ctx)
-
 #elif CORO_ASM
 
 struct coro_context {
-  void **sp; /* must be at offset 0 */
+  volatile void **sp;
 };
 
-void __attribute__ ((__noinline__, __regparm__(2)))
-coro_transfer (coro_context *prev, coro_context *next);
-
-# define coro_destroy(ctx) (void *)(ctx)
-
-#elif CORO_PTHREAD
-
-# include <pthread.h>
-
-extern pthread_mutex_t coro_mutex;
-
-struct coro_context {
-  pthread_cond_t cv;
-  pthread_t id;
-};
-
-void coro_transfer (coro_context *prev, coro_context *next);
-void coro_destroy (coro_context *ctx);
+void __attribute__ ((__noinline__, __fastcall__))
+     coro_transfer(coro_context *prev, coro_context *next);
 
 #endif
 
